﻿

''' <summary>
''' Este es el objeto que inicia el proceso
''' </summary>
Public Class Chain_Inicial
    Implements System.IDisposable

    ' '---------------------
    ' Este es el Objeto que llama el cliente que quiere hacer uso del patrón Chain
    ' Se encarga de:
    ' a) Definir todos los objetos de la cadena
    ' b) Dirigir la pregunta del cliente al primer eslabón de la misma
    ' c) También es el que devuelve el resultado al cliente

    ' Una observación
    ' Esta clase es la que mantiene la referencia a todos los objetos que componen los eslabones
    ' si va a usarse muchas veces solo debería instanciarse una vez en la aplicación
    ' porque hay un coste (importante) en la declaración y en la eliminación de los objetos.
    '
    ' Este proceso puede realizarse de muchas formas, pero esta es la que me parece mas sensata porque 
    ' todo el proceso de instanciación de objetos, definición de relaciones y montaje de la cadena y la
    ' eliminación de los mismos [Dispose], esta todo junto y es mas fácil de mantener
    ' Por otra parte, funciona a modo de un objeto [Facade], ocultando toda la estructura de clases,
    ' y mostrando únicamente al cliente, la función a la que tiene que llamar [ExecuteTarea]

    ' '---------------------
    ' Forma de uso
    ' '---------------------
    '  ' A) Definición de objetos
    '    Dim localObjChain As New Chain_Inicio
    '    Dim localObjMensajeVo As New MensajeVo
    '    Dim localObjRespuestaVo As New RespuestaVo
    ' '---------------------
    '  ' B) Carga de los valores de trabajo
    '    localObjMensajeVo.DatoUno = 90
    '    localObjMensajeVo.DatoDos = 5

    '  '---------------------
    '  ' C) Operación a realizar
    '    localObjMensajeVo.Key = "suma"
    '  ' D) Iniciar la tarea de la cadena y obtener el resultado
    '    localObjRespuestaVo = localObjChain.ExecuteTarea(localObjMensajeVo)
    '------------------
    ' /Eof forma de uso
    '------------------


    '---------------------------------------------------------
    '  ' A) Definición de objetos
    Private campoEslabonSuma As EslabonSuma
    Private campoEslabonResta As EslabonResta
    Private campoEslabonMultiplicacion As EslabonMultiplicacion
    Private campoEslabonDivision As EslabonDivision
    Private campoNodoCentinela As Eslabon_Centinela


    Public Sub New()
        '----------------------------
        ' Instanciar los eslabones
        campoEslabonSuma = New EslabonSuma
        campoEslabonResta = New EslabonResta
        campoEslabonMultiplicacion = New EslabonMultiplicacion
        campoEslabonDivision = New EslabonDivision
        campoNodoCentinela = New Eslabon_Centinela
        '----------------------------
        ' Decir quien es el siguiente elemento de la cadena
        campoEslabonSuma.EstablecerSiguienteEslabon(campoEslabonResta)
        campoEslabonResta.EstablecerSiguienteEslabon(campoEslabonMultiplicacion)
        campoEslabonMultiplicacion.EstablecerSiguienteEslabon(campoEslabonDivision)
        campoEslabonDivision.EstablecerSiguienteEslabon(campoNodoCentinela)


        ' En el nodo division no hace falta decir quien es el hijo,
        ' porque al crearlo, se coloca automáticamente el nodo centinela

    End Sub


    Public Function ExecuteTarea(ByRef paramObjUnMensaje As MensajeVo) As RespuestaVo

        ' Pasar el problema al PRIMER  eslabón de la cadena
        Return Me.campoEslabonSuma.ExecuteTarea(paramObjUnMensaje)

    End Function


#Region "IDisposable Support.   [Para clases normales y bases ] Implementación de .NET Version: [2020/04/29]"

    ''' <summary>
    '''  Variable local para detectar llamadas redundantes que indica [True] el objeto se ha eliminado, [False], en caso contrario. 
    ''' </summary>
    Private campoDisposedValue As System.Boolean

    ''' <summary>
    '''     Obtiene un valor que indica si el objeto se ha eliminado. 
    ''' </summary>
    ''' <value>
    '''   <para>Tipo: <see cref="System.Boolean">Boolean</see></para>
    '''   <para>Es true si objeto se ha eliminado; en caso contrario, es false.</para>
    ''' </value>
    Public Overloads ReadOnly Property IsDisposed() As System.Boolean
        Get
            Return campoDisposedValue
        End Get
    End Property


    ''' <summary>
    '''  [Dispose] Código de limpieza de la clase
    ''' </summary>
    ''' <param name="paramQuienMeLLama">[True], llama el usuario, [False], llama el sistema</param>
    ''' <remarks>
    '''  <para>
    '''     Dispose (bool quienMeLLama) se ejecuta en dos escenarios distintos.
    '''  </para>
    '''  <para>
    '''     Si [quienMeLLama] es igual a true, el método ha sido llamado 
    '''     directamente, o indirectamente, por el código de un usuario. 
    '''     Los recursos administrados y no administrados se pueden eliminar.
    '''  </para>
    '''  <para>
    '''     Si [quienMeLLama] es igual a false, el método ha sido llamado por el
    '''     [runtime] desde el interior del finalizador y usted no debe hacer 
    '''     referencia a otros objetos, ni siquiera para eliminarlos.
    '''     Solo los recursos no administrados pueden ser eliminados.
    '''  </para>
    ''' </remarks>
    Protected Overridable Sub Dispose(paramQuienMeLLama As System.Boolean)
        If Me.campoDisposedValue = False Then
            Me.campoDisposedValue = True
            If paramQuienMeLLama Then
                '------------------------------------------------------------------------------------------
                ' 1) Desechar objetos SI administrados.
                If Not (campoEslabonSuma Is Nothing) Then
                    campoEslabonSuma.Dispose()
                    campoEslabonSuma = Nothing
                End If
                If Not (campoEslabonResta Is Nothing) Then
                    campoEslabonResta.Dispose()
                    campoEslabonResta = Nothing
                End If
                If Not (campoEslabonMultiplicacion Is Nothing) Then
                    campoEslabonMultiplicacion.Dispose()
                    campoEslabonMultiplicacion = Nothing
                End If
                If Not (campoEslabonDivision Is Nothing) Then
                    campoEslabonDivision.Dispose()
                    campoEslabonDivision = Nothing
                End If
                If Not (campoNodoCentinela Is Nothing) Then
                    campoNodoCentinela.Dispose()
                    campoNodoCentinela = Nothing
                End If
            End If
            '------------------------------------------------------------------------------------------
            ' 2) Liberar objetos NO administrados.
            ' 3) Establecer campos grandes como Null.
            '------------------------------------------------------------------------------------------
        End If
    End Sub


    ''' <summary>
    '''  [Dispose] Código de limpieza de la clase
    ''' </summary>
    ''' <remarks>
    '''  Visual Basic agregó este código para implementar correctamente el patrón descartable.
    '''  No cambie este código. Coloque el código de limpieza en Dispose(disposing As Boolean).
    ''' </remarks>
    Public Sub Dispose() Implements IDisposable.Dispose
        Me.Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
#End Region


End Class
